@@ -424,16 +424,16 @@ class DriverElement(DrissionElement):
     def input(self,
               vals: Union[str, tuple],
               clear: bool = True,
-              insure_input: bool = True,
+              insure: bool = True,
               timeout: float = None) -> bool:
         """输入文本或组合键，也可用于输入文件路径到input元素（文件间用\n间隔）                          \n
         :param vals: 文本值或按键组合
         :param clear: 输入前是否清空文本框
-        :param insure_input: 确保输入正确，解决文本框有时输入失效的问题，不能用于输入组合键
+        :param insure: 确保输入正确，解决文本框有时输入失效的问题，不能用于输入组合键
         :param timeout: 尝试输入的超时时间，不指定则使用父页面的超时时间，只在insure_input为True时生效
         :return: bool
         """
-        if not insure_input:  # 普通输入
+        if not insure or self.tag != 'input' or self.prop('type') != 'text':  # 普通输入
             if clear:
                 self.inner_ele.clear()
 
@@ -470,13 +470,52 @@ class DriverElement(DrissionElement):
                     return True
 
     def run_script(self, script: str, *args) -> Any:
-        """执行js代码，传入自己为第一个参数  \n
+        """执行js代码，代码中用arguments[0]表示自己    \n
         :param script: js文本
         :param args: 传入的参数
         :return: js执行结果
         """
         return self.inner_ele.parent.execute_script(script, self.inner_ele, *args)
 
+    def scroll_to(self, mode: str = 'bottom', pixel: int = 300) -> None:
+        """按参数指示方式滚动元素                                                                                    \n
+        :param mode: 可选滚动方向：'top', 'bottom', 'half', 'rightmost', 'leftmost', 'up', 'down', 'left', 'right'
+        :param pixel: 滚动的像素
+        :return: None
+        """
+        if mode == 'top':
+            self.run_script("arguments[0].scrollTo(arguments[0].scrollLeft,0);")
+
+        elif mode == 'bottom':
+            self.run_script("arguments[0].scrollTo(arguments[0].scrollLeft,arguments[0].scrollHeight);")
+
+        elif mode == 'half':
+            self.run_script("arguments[0].scrollTo(arguments[0].scrollLeft,arguments[0].scrollHeight/2);")
+
+        elif mode == 'rightmost':
+            self.run_script("arguments[0].scrollTo(arguments[0].scrollWidth,arguments[0].scrollTop);")
+
+        elif mode == 'leftmost':
+            self.run_script("arguments[0].scrollTo(0,arguments[0].scrollTop);")
+
+        elif mode == 'up':
+            pixel = pixel if pixel >= 0 else -pixel
+            self.run_script(f"arguments[0].scrollBy(0,{pixel});")
+
+        elif mode == 'down':
+            self.run_script(f"arguments[0].scrollBy(0,{pixel});")
+
+        elif mode == 'left':
+            pixel = pixel if pixel >= 0 else -pixel
+            self.run_script(f"arguments[0].scrollBy({pixel},0);")
+
+        elif mode == 'right':
+            self.run_script(f"arguments[0].scrollBy({pixel},0);")
+
+        else:
+            raise ValueError("mode参数只能是'top', 'bottom', 'half', 'rightmost', "
+                             "'leftmost', 'up', 'down', 'left', 'right'。")
+
     def submit(self) -> Union[bool, None]:
         """提交表单"""
         try:
