@@ -31,6 +31,7 @@ def show_settings(ini_path: str = None) -> None:
 
 def set_paths(driver_path: str = None,
               chrome_path: str = None,
+              local_port: Union[int, str] = None,
               debugger_address: str = None,
               tmp_path: str = None,
               download_path: str = None,
@@ -41,6 +42,7 @@ def set_paths(driver_path: str = None,
     """快捷的路径设置函数                                          \n
     :param driver_path: chromedriver.exe路径
     :param chrome_path: chrome.exe路径
+    :param local_port: 本地端口号
     :param debugger_address: 调试浏览器地址，例：127.0.0.1:9222
     :param download_path: 下载文件路径
     :param tmp_path: 临时文件夹路径
@@ -53,7 +55,7 @@ def set_paths(driver_path: str = None,
     om = OptionsManager(ini_path)
 
     def format_path(path: str) -> str:
-        return '' if not path else str(path).replace('/', '\\')
+        return path or ''
 
     if driver_path is not None:
         om.set_item('paths', 'chromedriver_path', format_path(driver_path))
@@ -61,6 +63,9 @@ def set_paths(driver_path: str = None,
     if chrome_path is not None:
         om.set_item('chrome_options', 'binary_location', format_path(chrome_path))
 
+    if local_port is not None:
+        om.set_item('chrome_options', 'debugger_address', format_path(f'127.0.0.1:{local_port}'))
+
     if debugger_address is not None:
         om.set_item('chrome_options', 'debugger_address', format_path(debugger_address))
 
@@ -323,7 +328,7 @@ def _get_chrome_version(path: str) -> Union[str, None]:
     try:
         return (popen(f'wmic datafile where "name=\'{path}\'" get version').read()
                 .lower().split('\n')[2].replace(' ', ''))
-    except:
+    except Exception:
         return None
 
 
@@ -346,18 +351,24 @@ def _download_driver(version: str, save_path: str = None, show_msg: bool = True)
     except ValueError:
         return None
 
-    for i in page.eles('xpath://pre/a'):
-        remote_main = i.text.split('.')[0]
+    remote_versions = page.eles(f'xpath://pre/a[starts-with(text(),"{loc_main}.")]')
+    remote_versions = [v for v in remote_versions if v.text and v.text[-1] == '/']
+    remote_versions.sort(key=lambda x: x.text)
 
+    for i in remote_versions:
         try:
             remote_num = int(sub(r'[./]', '', i.text))
         except ValueError:
             continue
 
-        if remote_main == loc_main and remote_num >= loc_num:
+        if remote_num >= loc_num:
             remote_ver = i.text
             break
 
+    # 没有匹配到，则取大版本的最后一个号
+    if remote_versions and not remote_ver:
+        remote_ver = remote_versions[-1].text
+
     if remote_ver:
         url = f'https://cdn.npm.taobao.org/dist/chromedriver/{remote_ver}chromedriver_win32.zip'
         save_path = save_path or Path(__file__).parent
